c programma per costruire mappe a copertura esagonale con
c beam circolari.
c input : ra, dec 			[gradi]
c	    coordtype
c	    prmfile
c
c output: fort.66 per smongo (serp.mac), copertura e ordine di mapping
c         fort.77 lista delle coord. eq. dei punti di mappa
c 
c
	implicit double precision (a-h,o-z)
c
	character*100 prmfile
	character*20 ra_ch, dec_ch
      integer nargs
c
c     Parameters
c
	include 'esagonmap.inc'
c
c
c	formats
c
100	format(f8.4)
101	format(f7.4)
c
	nargs = iargc()
	if(nargs.lt.4) then
		write(*,*) 'Usage: esagonmap x y  coordtype prmfile'
		write(*,*) 'coordtype: coordinate system [EQU | GAL]'
		write(*,*) 'prmfile  : name of the configuration file'
		stop
	else
		call getarg(1,ra_ch)
		call getarg(2,dec_ch)
		call getarg(3,coordtype)
		call getarg(4,prmfile)
	end if
	read(ra_ch,*) ra0
	read(dec_ch,*) dec0
c
c----------------------------------------------------------------------
c        Reads the parameter file
c----------------------------------------------------------------------
      call read_prmfile(prmfile, ios)
      if(ios.lt.-1) STOP 'Read param file'
c
c	computes the coordinate of the map origin x0, yo [arcmin]
c
	x0 = ra0*60.0 - xoff
	y0 = dec0*60.0 - yoff
	call GridParams()
	call printparams(ra0, dec0)
	pa=pa/180.*PI
	np=0
c
	do ic=1,nc            ! nc defined in easgonmap.inc
		call evenrow(ic-1, np)
		call oddrow(ic-1,np)
	end do
c
	stop
	end
c
	subroutine printparams(ra0, dec0)
	real*8 ra0, dec0
	call printmap(ra0, dec0)
	call printgrid()
	return
	end
c
	subroutine printmap(ra0, dec0)
	real*8 ra0, dec0
c
	include 'esagonmap.inc'
c
	write(*,100)
	write(*,101) ra0, dec0
	write(*,102) wx, wy
	write(*,103) xoff, yoff
	write(*,104) pa, beam
100	format(1x,'Input parameters of the map')
101	format(1x,'Target coord: x = ',f7.2,' degree','   y = ',f7.2,' degree')
102	format(1x,'Dimensions:   x = ',f7.2,' arcmin','   y = ',f7.2,' arcmin')
103	format(1x,'Target pos:   x = ',f7.2,' arcmin','   y = ',f7.2,' arcmin')
104	format(1x,'Position ang.   = ',f7.2,' degree',' HPBW = ',f5.2,' arcmin')
	return
	end
c
      subroutine beampos(x,y,pa,x0,y0)
c
c     This subroutine computes the position (x0,y0) of the beam
c     corrected for the position angle
c
	implicit double precision(a-h,o-z)
	rho=sqrt(x**2+y**2)
	phi=atan2(y,x)
	phi0=phi+pa
	x0=rho*cos(phi0)
	y0=rho*sin(phi0)
	return
      end
c
      subroutine evenrow(ic,np)
	integer ic        ! current couple of rows
	integer ir        ! current row
c
c     Calcola le posizioni dei centri per le righe pari
c
	implicit double precision(a-h,o-z)
	real*8 xgrid, ygrid
	real*8 xbeam, ybeam
	include 'esagonmap.inc'
c
	ir = 2*ic
	ygrid = ir*dy + y0          ! [arcmin]
	do icol=1, ncol
		xgrid = icol*dx + x0  ! [arcmin]
      	call beampos(xgrid,ygrid,pa,xbeam,ybeam)
		np = np+1
		call output(np,ir,icol,xbeam,ybeam)
	end do
      return
      end
c	
      subroutine oddrow(ic,np)
	integer ic        ! current couple of rows
	integer ir        ! current row
c
c     Calcola le posizioni dei centri per le righe dispari
c
	implicit double precision(a-h,o-z)
	real*8 xgrid, ygrid
	real*8 xbeam, ybeam
c
	include 'esagonmap.inc'
	ir = 2*ic + 1
	ygrid = ir*dy + y0             ! [arcmin]
	do icol=1, ncol
		xgrid = icol*dx + a + x0 ! [arcmin]
      	call beampos(xgrid,ygrid,pa,xbeam,ybeam)
		np = np+1
		call output(np,ir,icol,xbeam,ybeam)
	end do
      return
      end
c	
	subroutine output(np,irow,icol,x,y)
	character extension*40
	implicit double precision(a-h,o-z)
	double precision lii, bii
	double precision ra, dec
      integer mlii
	integer rah,ram
	integer decd,decm
	integer ras, decs
c
	include 'esagonmap.inc'
c
      parameter(raddeg = 180.0/3.1415927)
c
102   format("'",I5.5,SP,I3.3,"' ",F7.1," ")
c23456789012345678901234567890123456789012345678901234567890123456789012
c
c     xdeg, ydeg coordinate in gradi
c
	xdeg = x/60.0
	ydeg = y/60.0
      mlii = xdeg*100
      mbii = ydeg*10
	write(extension,102)mlii,mbii, vlsr
      if(index(coordtype,"GAL").eq.1) then
            lii = xdeg/raddeg
            bii = ydeg/raddeg
		call gal2eq(lii,bii,ra,dec)
c
c	si riconverte in gradi
c
            ra = ra * raddeg
            if(ra .lt. 0.0) then
                ra = ra + 360.0
            end if
            dec = dec * raddeg
	else
		ra  = xdeg
		dec = ydec
	end if
	call deg2dmis(ra/15.0,rah,ram,ras)
	call deg2dmis(dec,decd,decm,decs)
c	
	write(6,'(2i6,2f10.2)') irow,icol,ra,dec
	write(66,'(3i6,2f10.2)') np,irow,icol,ra,dec
	write(77,'(2i2.2,i5.5,1x,i3.2,i2.2,i5.5,1x,A)')
     +	        rah,ram,ras,decd,abs(decm),abs(decs),extension
	return
	end
c
c
	subroutine deg2dmis(xa,xah,xam,xas)
	real*8  xa
	integer xah,xam,xas
c
C     Questa subroutine si aspetta di lavorare in gradi
c
	xah=xa
	xam=(xa-xah)*60.0
	as=(xa-(xah+xam/60.0))*3600.0
      xas = as*1000
	return
	end
c
	subroutine err_message(message)
	character*(*) message
c
	write(*,*) message
	return
	end
