c Why this module.
c
c  Many programs uses one or more parameters to adapt to external conditions.
c  Usually parameters can be specified in the run string, but if their number
c  is to large this solution is no longer convenient.
c
c  As an alternative, parameters can be specified in a configuration file
c  whose name is passed to the program via the run string as well.
c
c  To allow the parameters to be specifiec in any order, it is convenient
c  to identify the parameters with a keyword. 
c
c  This module assumes that each record of the configuration file contains 
c  a keyword and the values of the associated parameters, separated by the
c  '=' character.
c  
c  With this assumption, this module provide a set of subroutines/functions
c  that returns the keyword and the string of parameter values, making no
c  operations on them.
c
c  It is responsibility of the calling program to identify the keyword
c  and read the parameter values from the string. This is done by providing
c  the subroutine PF_AssignPrm, whose syntax is the following:
c
c  subroutine PF_AssignPrm(keyword, params, ios)
c  character*(*) keyword, params
c  integer ios   ! error code-> -3 keyword error
c                                0 no error
c
c  The calling program must also provide the subroutine:
c
c  subroutine err_message(message)
c  character *(*) message
c
c  that prints the character string message somewhere.
c
c  An example of this subroutine follows:
c
c  This module is called through the statement:
c
c  call read_prmfile(prmfile, ios)
c  character*(*) prmfile ! The ASCII configuration file 
c  int ios               ! The I/O error code
c
C=======================================================================
c     SUBROUTINE read_prmfile(prmfile,ios)
c
c     This subroutine reads  the parameter values specified in the
c     configuration file prmfile.
c
c     ios = error parameter
c         = -1 EOF found in file
c         = -2 ERR found in any I/O operation
c         = -3 Keyword error
c
C-----------------------------------------------------------------------
      subroutine read_prmfile(prmfile, ios)
      character*(*) prmfile
      integer ios
      character*120 line
      character*20 keyword, get_keyword
      character*100 message

      ios = 0
      open (12, file=prmfile, status="OLD", ERR=300, IOSTAT=ios)
10    read(12,'(A)',END=190, IOSTAT=ios) line
      keyword = get_keyword(line, nchprm)
	call FP_AssignPrm(keyword(:LNBLNK(keyword)), line(nchprm:), ios )
	if(ios.eq.-3) goto 190  ! break loop
      go to 10
c
190   close(12)
      return
300   message = ' opening' // prmfile
      call err_message(message)
      return
      end

C=======================================================================
c     character get_keyword(line, nchar)
c
c     This function returns the keyword and the param substring position 
c     in line
c-----------------------------------------------------------------------
c
      character*20 function get_keyword(line, nchar)
      character*(*) line

      n = index(line,"=")
      if (n.gt.0) then
          get_keyword = line(:n-1)
          nchar = n + 1
      else
          n = index(line," ")
          if (n.eq.0) then
             message = ' time separation param: syntax error' 
             call err_message(message)
          else
             get_keyword = line(:n)
             nchar = n + 1
          end if
      end if
      return
      end

